from fastapi import APIRouter, Depends, HTTPException, Response
from sqlalchemy.orm import Session

from src.api.cruds import subscription as sub_crud
from src.api.db import get_db
from src.api.schemas.subscription import Subscription, SubscriptionRemove

router = APIRouter()


@router.post("")
def create_subscription(subscription: Subscription, db: Session = Depends(get_db)):
    return sub_crud.create_subscription(db=db, subscription=subscription)


@router.get("/{user_id}", response_model=list[Subscription])
def get_subscriptions(user_id: int, db: Session = Depends(get_db)):
    db_subscriptions = sub_crud.get_subscriptions(db=db, user_id=user_id)
    if db_subscriptions is None:
        raise HTTPException(status_code=404, detail="Subscription not found")
    return db_subscriptions


@router.delete("/{user_id}")
def delete_subscription(
    user_id: int, subscription: SubscriptionRemove, db: Session = Depends(get_db)
):
    sub_crud.remove_subscription(
        db=db, user_id=user_id, flight_id=subscription.flight_id
    )
    return Response(status_code=204)
